clear all;

% This program recovers algebraically the last row or the last column of
% a Mueller matrix assuming no depolarization. 
% This code is free to use and modify but please cite the following two papers:
%(1) O. Arteaga and R. Ossikovski, J. Opt. Soc. Am. A 36, 416-427 (2019)
%(2) R. Ossikovski and O. Arteaga. J. of Vac. Sc. & Tech. B 37, 052905 (2019).

% Set a 12-element partial Mueller matrix as an input. Fill the missing row/column with 'i'. Eg:
M = [1 -0.43994 0.42106 i; -0.43369 0.76721 -0.11274 i; -0.42285 0.11251 -0.88226 i; 0.11845 -0.44815 -0.00734 i];
% alternatively: M=[1 -0.43994 0.42106 0.11735; -0.43369 0.76721 -0.11274 -0.44863; -0.42285 0.11251 -0.88226 0.01296; i i i i];

% check for row or column missing
if isreal(M(1, 4))
    f = 1; % row missing   
    M = transpose(M);
else
    f = 0; % column missing
end

% four 3 X 3 minors
m1 = M(2, 1) .* M(3, 2) .* M(4, 3) + M(2, 2) .* M(3, 3) .* M(4, 1) + M(2, 3) .* M(3, 1) .* M(4, 2);
m1 = m1 - (M(2, 3) .* M(3, 2) .* M(4, 1) + M(2, 1) .* M(3, 3) .* M(4, 2) + M(2, 2) .* M(3, 1) .* M(4, 3));

m2 = M(1, 1) .* M(3, 2) .* M(4, 3) + M(1, 2) .* M(3, 3) .* M(4, 1) + M(1, 3) .* M(3, 1) .* M(4, 2);
m2 = m2 - (M(1, 3) .* M(3, 2) .* M(4, 1) + M(1, 1) .* M(3, 3) .* M(4, 2) + M(1, 2) .* M(3, 1) .* M(4, 3));

m3 = M(1, 1) .* M(2, 2) .* M(4, 3) + M(1, 2) .* M(2, 3) .* M(4, 1) + M(1, 3) .* M(2, 1) .* M(4, 2);
m3 = m3 - (M(1, 3) .* M(2, 2) .* M(4, 1) + M(1, 1) .* M(2, 3) .* M(4, 2) + M(1, 2) .* M(2, 1) .* M(4, 3));

m4 = M(1, 1) .* M(2, 2) .* M(3, 3) + M(1, 2) .* M(2, 3) .* M(3, 1) + M(1, 3) .* M(2, 1) .* M(3, 2);
m4 = m4 - (M(1, 3) .* M(2, 2) .* M(3, 1) + M(1, 1) .* M(2, 3) .* M(3, 2) + M(1, 2) .* M(2, 1) .* M(3, 3));

% normalized (and signed) minors
sqrtm = sqrt(m4 .* m4 + m3 .* m3 + m2 .* m2 - m1 .* m1);
mn1 = m1 ./ sqrtm;
mn2 = m2 ./ sqrtm;
mn3 = -m3 ./ sqrtm;
mn4 = m4 ./ sqrtm;

% square root of det(M), averages the three theoretically equivalent expressions
sqrtdet = M(1, 1) .* M(1, 1) - M(2, 1) .* M(2, 1) - M(3, 1) .* M(3, 1) - M(4, 1) .* M(4, 1);
sqrtdet = sqrtdet - (M(1, 2) .* M(1, 2) - M(2, 2) .* M(2, 2) - M(3, 2) .* M(3, 2) - M(4, 2) .* M(4, 2));
sqrtdet = sqrtdet - (M(1, 3) .* M(1, 3) - M(2, 3) .* M(2, 3) - M(3, 3) .* M(3, 3) - M(4, 3) .* M(4, 3));
sqrtdet = sqrtdet / 3;
% fourth root of det(M)
qrdet = sqrt(sqrtdet);

% recovered fourth column
M(1,4) = qrdet .* mn1;
M(2,4) = qrdet .* mn2;
M(3,4) = qrdet .* mn3;
M(4,4) = qrdet .* mn4;

% if row missing
if f == 1 
    M = transpose(M);
end

% print out the result 
fprintf('The recovered Mueller matrix is \n')
fprintf ('%1.4f %1.4f %1.4f %1.4f \n', M.') 
